@@ -6,13 +6,16 @@
 '''zoort
 
 Usage:
-  zoort backup <database> [--path=<path>] [--upload_s3=<s3>] [--upload_glacier=<glacier>] [--encrypt=<encrypt>]
+  zoort backup <database> [--path=<path>] [--upload_s3=<s3>] [--upload_glacier=<glacier>] [--upload_dropbox=<dropbox>] [--upload_swift=<swift>] [--encrypt=<encrypt>]
   zoort backup <database> <user> <password> [--path=<path>] [--upload_s3=<s3>] [--upload_glacier=<glacier>] [--encrypt=<encrypt>]
   zoort backup <database> <user> <password> <host> [--path=<path>] [--upload_s3=<s3>] [--upload_glacier=<glacier>] [--encrypt=<encrypt>]
   zoort backup_all [--auth=<auth>] [--path=<path>] [--upload_s3=<s3>] [--upload_glacier=<glacier>] [--encrypt=<encrypt>]
   zoort download_all
   zoort decrypt <path>
   zoort configure
+  zoort configure-aws
+  zoort configure-dropbox
+  zoort configure-swift
   zoort --version
   zoort --help
 
@@ -34,8 +37,10 @@ import dateutil.parser
 import boto
 import shutil
 import ftplib
+import dropbox
 from boto.s3.key import Key
 from docopt import docopt
+from swiftclient import Connection, ClientException, get_keystoneclient_2_0
 from functools import wraps
 from fabric.api import local, hide
 from fabric.colors import blue, red, green
@@ -50,7 +55,7 @@ try:
 except NameError:
     pass
 
-__version__ = '0.1.7'
+__version__ = '0.1.8'
 __author__ = 'Yohan Graterol'
 __license__ = 'MIT'
 
@@ -151,7 +156,7 @@ def factory_uploader(type_uploader, *args, **kwargs):
             dif = DELETE_WEEKS * 7 * 24 * 60
 
             for key in bucket.list():
-                if get_diff_date(key.creation_date) >= dif:
+                if get_diff_date(key.last_modified) >= dif:
                     ret.append(key)
 
             return ret
@@ -220,7 +225,7 @@ def factory_uploader(type_uploader, *args, **kwargs):
 
         def get_file_from_time(self, time_old):
             return self.session.query(
-                self.File).filter(self.File.date_upload<=time_old)
+                self.File).filter(self.File.date_upload <= time_old)
 
         def download_all_backups(self):
             jobs = self.vault.list_jobs(completed=True)
@@ -253,29 +258,35 @@ def factory_uploader(type_uploader, *args, **kwargs):
                 self.session.delete(archive)
                 self.session.flush()
 
-
     class FTP(object):
-        
+
         def __init__(self, *args, **kwargs):
             super(FTP, self).__init__()
             self.__dict__.update(kwargs)
             config_data = get_config_json()
 
-            self.host = kwargs.get('host', 
-                        config_data.get('ftp').get('host'))
-            self.user = kwargs.get('user', 
-                        config_data.get('ftp').get('user'))
-            self.passwd = kwargs.get('passwd', 
-                        config_data.get('ftp').get('passwd'))
-            self.path = normalize_path(kwargs.get('path', \
-                        config_data.get('ftp').get('path')))
+            self.host = kwargs.get('host', config_data.get('ftp').get('host'))
+            self.user = kwargs.get('user',
+                                   config_data.get('ftp').get('user'))
+            self.passwd = kwargs.get('passwd',
+                                     config_data.get('ftp').get('passwd'))
+            self.path = normalize_path(kwargs.get('path',
+                                       config_data.get('ftp').get('path')))
+
+            self.host = kwargs.get('host',
+                                   config_data.get('ftp').get('host'))
+            self.user = kwargs.get('user',
+                                   config_data.get('ftp').get('user'))
+            self.passwd = kwargs.get('passwd',
+                                     config_data.get('ftp').get('passwd'))
+            self.path = normalize_path(kwargs.get('path',
+                                       config_data.get('ftp').get('path')))
 
             self.name_backup = kwargs.get('name_backup', None)
 
             if not self.name_backup:
                 raise SystemExit(_error_codes.get(103))
-        
-        
+
         def connect(self):
             try:
                 self.conn = ftplib.FTP(self.host, self.user, self.passwd)
@@ -284,18 +295,15 @@ def factory_uploader(type_uploader, *args, **kwargs):
 
             print('Connected to {0}'.format(self.host))
 
-        
         def disconnect(self):
             self.conn.quit()
 
-
         def mkdir(self, dirname):
             try:
                 self.conn.mkd(dirname)
             except Exception, e:
-                raise SystemExit(_error_codes.get(13).
-                                    format(dirname, self.conn.pwd(), e))
-
+                raise SystemExit(_error_codes.get(13).format(
+                    dirname, self.conn.pwd(), e))
 
         def change_dir(self, dirname):
             try:
@@ -303,15 +311,13 @@ def factory_uploader(type_uploader, *args, **kwargs):
             except Exception, e:
                 raise SystemExit(_error_codes.get(14).format(dirname, e))
 
-        
         def send_file(self, filename):
             try:
                 backup_file = open(filename, 'rb')
                 self.conn.storbinary('STOR ' + filename, backup_file)
             except Exception, e:
                 raise SystemExit(_error_codes.get(15).
-                                    format(filename, path, e))
-
+                                 format(filename, self.path, e))
 
         def delete_file(self, filename):
             try:
@@ -319,7 +325,6 @@ def factory_uploader(type_uploader, *args, **kwargs):
             except Exception, e:
                 raise SystemExit(_error_codes.get(16).format(filename, e))
 
-
         def get_file_date(self, filename):
             try:
                 mdtm = self.conn.sendcmd('MDTM ' + filename)
@@ -328,13 +333,12 @@ def factory_uploader(type_uploader, *args, **kwargs):
 
             return mdtm[4:]
 
-
         def list_files(self):
             '''
             Return all files in the actual directory without '.' and '..'
             '''
             ret = []
-            
+
             for path in self.conn.nlst():
                 if path in ['.', '..']:
                     continue
@@ -342,13 +346,12 @@ def factory_uploader(type_uploader, *args, **kwargs):
 
             return ret
 
-        
         def goto_path(self, path):
             '''
             Change to 'path' directory or create if not exist
             '''
             try:
-                self.conn.cwd(folder)
+                self.conn.cwd(self.folder)
             except:
                 self.change_dir('/')
                 for folder in path.split('/'):
@@ -357,25 +360,24 @@ def factory_uploader(type_uploader, *args, **kwargs):
 
                     if not folder in self.conn.nlst():
                         self.mkdir(folder)
-                    
+
                     self.change_dir(folder)
 
-        
         def upload(self):
             self.connect()
-            
-            path = normalize_path(self.path) + 'week-' \
-                    + str(datetime.datetime.now().isocalendar()[1])
+
+            path = (normalize_path(self.path) + 'week-' +
+                    str(datetime.datetime.now().isocalendar()[1]))
             self.goto_path(path)
 
-            print('Uploading file to {0} in {1}'.format(self.host, self.conn.pwd()))
+            print('Uploading file to {0} in {1}'.format(self.host,
+                                                        self.conn.pwd()))
             name_backup = self.name_backup.split('/')[-1]
             self.send_file(name_backup)
             self.delete()
 
             self.disconnect()
 
-        
         def delete(self):
             global DELETE_BACKUP
 
@@ -385,7 +387,6 @@ def factory_uploader(type_uploader, *args, **kwargs):
             for filename in self._get_old_backup():
                 self.delete_file(filename)
 
-
         def _get_old_backup(self):
             global DELETE_WEEKS
             ret = []
@@ -397,18 +398,93 @@ def factory_uploader(type_uploader, *args, **kwargs):
                 self.change_dir(path)
 
                 for backup in self.list_files():
-                    if get_diff_date(self.get_file_date(backup))  >= dif:
+                    if get_diff_date(self.get_file_date(backup)) >= dif:
                         # Add full path of backup
                         ret.append(self.conn.pwd() + '/' + backup)
-                
+
                 self.change_dir('..')
 
             return ret
 
+    class DropboxStorage(object):
+
+        def __init__(self, *args, **kwargs):
+            super(DropboxStorage, self).__init__()
+            self.__dict__.update(kwargs)
+            config_data = get_config_json()
+
+            self.auth_token = config_data.get('dropbox').get('auth_token')
+            self.path = config_data.get('dropbox').get('path')
+            self.name_backup = kwargs.get('name_backup', None)
+            self.client = dropbox.client.DropboxClient(self.auth_token)
+            if not self.name_backup:
+                raise SystemExit(_error_codes.get(103))
+
+        def send_file(self, filename):
+            try:
+                backup_file = open(filename, 'rb')
+                response = self.client.put_file(filename, backup_file)
+                print('Uploading file {0} to directory "{1}" on Dropbox'.
+                      format(filename,
+                             response.get('root')))
+            except Exception, e:
+                raise SystemExit(_error_codes.get(115).format(
+                                 filename, 'Dropbox', e))
+
+        def upload(self):
+            name_backup = self.name_backup.split('/')[-1]
+            self.send_file(name_backup)
+
+    class SwiftStorage(object):
+        def __init__(self, *args, **kwargs):
+            super(SwiftStorage, self).__init__()
+            self.__dict__.update(kwargs)
+            config_data = get_config_json()
+            # Load config variables.
+            self.auth_url = config_data.get('swift').get('auth_url')
+            self.access_key = config_data.get('swift').get('access_key')
+            self.secret_key = config_data.get('swift').get('secret_key')
+            self.auth_version = config_data.get('swift').get('auth_version')
+            self.tenant_name = config_data.get('swift').get('tenant_name')
+            self.insecure = True
+            self.container = config_data.get('swift').get('container')
+            self.name_backup = kwargs.get('name_backup', None)
+            self.conn = Connection(
+                authurl=self.auth_url,
+                user=self.access_key,
+                key=self.secret_key,
+                auth_version=self.auth_version,
+                tenant_name=self.tenant_name,
+                insecure=self.insecure)
+            try:
+                self.conn.head_container(self.container)
+            except:
+                self.conn.put_container(self.container)
+
+            if not self.name_backup:
+                raise SystemExit(_error_codes.get(103))
+
+        def send_file(self, filename, **kwargs):
+            try:
+                backup_file = open(filename, 'rb')
+                response = self.conn.put_object(
+                    self.container, filename, backup_file)
+                print('Uploading file {0} to container "{1}" on swift'.
+                      format(filename,
+                             self.container))
+            except Exception, e:
+                raise SystemExit(_error_codes.get(115).format(
+                                 filename, 'Swift', e))
+
+        def upload(self):
+            name_backup = self.name_backup.split('/')[-1]
+            self.send_file(name_backup)
 
     uploaders = {'S3': AWSS3,
                  'Glacier': AWSGlacier,
-                 'FTP': FTP}
+                 'FTP': FTP,
+                 'Dropbox': DropboxStorage,
+                 'Swift': SwiftStorage}
 
     upload = uploaders.get(type_uploader)(*args, **kwargs)
 
@@ -447,6 +523,7 @@ def get_input(msg, is_password=False, verify_type=None):
 
 
 def get_config_json():
+    config = None
     try:
         config = open('/etc/zoort/config.json')
     except IOError:
@@ -462,7 +539,7 @@ def get_config_json():
     return config_data
 
 
-def configure():
+def configure(service=None):
     print('''
     Zoort v-{0}
     Please fill all fields for configure Zoort.
@@ -474,14 +551,10 @@ def configure():
     config_dict['admin_user'] = get_input('MongoDB User Admin: ')
     config_dict['admin_password'] = \
         get_input('MongoDB Password Admin (Is hidden): ', True)
-    # Define dict to aws key
-    config_dict['aws'] = dict()
 
-    # AWS Variables
-    config_dict['aws']['aws_access_key'] = \
-        get_input('AWS Access Key (Is hidden): ', True)
-    config_dict['aws']['aws_secret_key'] = \
-        get_input('AWS Secret Key (Is hidden): ', True)
+    if 'aws' in service:
+        # Define dict to aws key
+        config_dict['aws'] = dict()
 
     try:
         if int(get_input('Do you want use Amazon Web Services S3? '
@@ -494,7 +567,7 @@ def configure():
                 get_input('AWS Vault Glacier name: ')
         config_dict['aws']['aws_key_name'] = \
             get_input('Key name for backups file: ')
-        config_dict['aws']['password_file'] = \
+        config_dict['password_file'] = \
             get_input('Password for encrypt with AES (Is hidden): ', True)
         config_dict['delete_backup'] = \
             int(get_input('Do you want delete old backups? '
@@ -503,6 +576,49 @@ def configure():
             config_dict['delete_weeks'] = \
                 get_input('When weeks before of backups do you want delete? '
                           '(Number please) ', verify_type=int)
+
+        if 'dropbox' in service:
+            # Define dict to dropbox key
+            config_dict['dropbox'] = dict()
+            # Dropbox Variables
+            try:
+                config_dict['dropbox']['app_key'] = \
+                    get_input('Dropbox app key: ')
+                config_dict['dropbox']['secret_key'] = \
+                    get_input('Dropbox secret key: ')
+                flow = dropbox.client.DropboxOAuth2FlowNoRedirect(
+                    config_dict['dropbox']['app_key'],
+                    config_dict['dropbox']['secret_key'])
+                authorize_url = flow.start()
+                config_dict['dropbox']['auth_code'] = \
+                    get_input('Go to ' + authorize_url + ', allow,' +
+                              'an put the code here: ')
+                access_token, user_id = flow.finish(
+                    config_dict['dropbox']['auth_code'])
+                config_dict['dropbox']['auth_token'] = \
+                    get_input('This is your access token ' + access_token +
+                              ' Type the code: ')
+            except ValueError:
+                raise SystemExit(_error_codes.get(108))
+        if 'swift' in service:
+            # Define dict to swift
+            config_dict['swift'] = dict()
+            # swift variables
+            try:
+                config_dict['swift']['auth_url'] = \
+                    get_input('Swift Auth url: ')
+                config_dict['swift']['access_key'] = \
+                    get_input('Swift username: ')
+                config_dict['swift']['secret_key'] = \
+                    get_input('Swift password: ')
+                config_dict['swift']['auth_version'] = \
+                    get_input('Swift version auth used: ')
+                config_dict['swift']['tenant_name'] = \
+                    get_input('Swift tenant used: ')
+                config_dict['swift']['container'] = \
+                    get_input('Swift container used: ')
+            except ValueError:
+                raise SystemExit(_error_codes.get(108))
     except ValueError:
         raise SystemExit(_error_codes.get(108))
 
@@ -540,8 +656,8 @@ def load_config(func):
             AWS_KEY_NAME = config_data.get('aws').get('aws_key_name')
             DELETE_BACKUP = config_data.get('delete_backup')
             DELETE_WEEKS = config_data.get('delete_weeks')
-        except ValueError:
-            pass
+        except ValueError, e:
+            print(e)
         return func(*args, **kwargs)
     return wrapper
 
@@ -555,14 +671,14 @@ def normalize_path(path):
     return path
 
 
-def compress_folder_dump(path):
+def compress_folder_dump(path, target):
     '''
     Compress folder dump to tar.gz file
     '''
     import tarfile
     if not path or not os.path.isdir(path):
         raise SystemExit(_error_codes.get(105))
-    name_out_file = ('dump-' +
+    name_out_file = (target + 'dump-' +
                      datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S'))
     tar = tarfile.open(name_out_file + '.tar.gz', 'w:gz')
     tar.add(path, arcname='dump')
@@ -574,7 +690,6 @@ def encrypt_file(path, output, password=None):
     '''
     Encrypt file with AES method and password.
     '''
-    global PASSWORD_FILE
     if not password:
         password = PASSWORD_FILE
     query = 'openssl aes-128-cbc -salt -in {0} -out {1} -k {2}'
@@ -604,9 +719,8 @@ def optional_actions(encrypt, path, compress_file, **kwargs):
     yes = ('y', 'Y')
     file_to_upload = normalize_path(path) + compress_file[1]
     if encrypt in yes:
-        encrypt_file(normalize_path(path) + compress_file[1],
-                     normalize_path(path) + compress_file[0])
-        file_to_upload = normalize_path(path) + compress_file[0]
+        encrypt_file(compress_file[1], compress_file[0])
+        file_to_upload = compress_file[0]
 
     if kwargs.get('s3') in yes:
         factory_uploader('S3', name_backup=file_to_upload,
@@ -618,6 +732,12 @@ def optional_actions(encrypt, path, compress_file, **kwargs):
                          path=os.path.join(os.path.expanduser('~'),
                                            '.zoort.db'),
                          action='upload')
+    if kwargs.get('dropbox') in yes:
+        factory_uploader('Dropbox', name_backup=file_to_upload,
+                         action='upload')
+    if kwargs.get('swift') in yes:
+        factory_uploader('Swift', name_backup=file_to_upload,
+                         action='upload')
 
 
 @load_config
@@ -631,7 +751,13 @@ def main():
     if args.get('decrypt'):
         decrypt_file(args.get('<path>'))
     if args.get('configure'):
-        configure()
+        configure(service='all')
+    if args.get('configure-aws'):
+        configure(service='aws')
+    if args.get('configure-dropbox'):
+        configure(service='dropbox')
+    if args.get('configure-swift'):
+        configure(service='swift')
     if args.get('download_all'):
         download_all()
 
@@ -648,9 +774,11 @@ def backup_database(args):
     password = args.get('<password>')
     database = args['<database>']
     host = args.get('<host>') or '127.0.0.1'
-    path = args.get('[--path]') or os.getcwd()
+    path = args.get('--path') or os.getcwd()
     s3 = args.get('--upload_s3')
     glacier = args.get('--upload_glacier')
+    dropbox = args.get('--upload_dropbox')
+    swift = args.get('--upload_swift')
     encrypt = args.get('--encrypt') or 'Y'
 
     if not database:
@@ -671,11 +799,15 @@ def backup_database(args):
                        database=database,
                        host=host,
                        path=path))
-    compress_file = compress_folder_dump(normalize_path(path) + 'dump')
+
+    compress_file = compress_folder_dump(
+        normalize_path(path) + 'dump', normalize_path(path))
 
     shutil.rmtree(normalize_path(path) + 'dump')
 
-    optional_actions(encrypt, path, compress_file, s3=s3, glacier=glacier)
+    optional_actions(encrypt, path, compress_file,
+                     s3=s3, glacier=glacier, dropbox=dropbox,
+                     swift=swift)
 
 
 def backup_all(args):
@@ -685,14 +817,19 @@ def backup_all(args):
     username = None
     password = None
     auth = args.get('--auth')
-    path = args.get('--path') or os.getcwd()
+    path = args.get('--path')
     s3 = args.get('--upload_s3')
     glacier = args.get('--upload_glacier')
+    dropbox = args.get('--upload_dropbox')
+    swift = args.get('--upload_swift')
     encrypt = args.get('--encrypt') or 'Y'
 
     if (ADMIN_USER and ADMIN_PASSWORD):
         username = ADMIN_USER
         password = ADMIN_PASSWORD
+
+    if not path:
+        path = os.getcwd()
     if path and not os.path.isdir(path):
         raise SystemExit(_error_codes.get(105))
 
@@ -707,11 +844,13 @@ def backup_all(args):
                        password=password,
                        path=path))
 
-    compress_file = compress_folder_dump(normalize_path(path) + 'dump')
+    compress_file = compress_folder_dump(
+        normalize_path(path) + 'dump', normalize_path(path))
 
     shutil.rmtree(normalize_path(path) + 'dump')
 
-    optional_actions(encrypt, path, compress_file, s3=s3, glacier=glacier)
+    optional_actions(encrypt, path, compress_file, s3=s3, glacier=glacier,
+                     dropbox=dropbox, swift=swift)
 
 
 if __name__ == '__main__':
